/*********************************************************************/
/*  Module Name           : legacyservice.c                          */
/*                                                                   */
/*  Environment           : Microsoft Visual C++                     */
/*                                                                   */
/*  Parameters            : Takes 2 arguments                        */
/*                           QueueManagerName                        */
/*                           QueueName                               */
/*                                                                   */
/*  Description : Sample program to get a messages from a queue and  */
/*                reply as follows                                   */
/*                - if input MessageType is IA81BUY send IA81CONF    */
/*                     with IA81CONF.Confirm set based on content of */
/*                     IA81BUY.ItemReference                         */ 
/*                - if input MessageType is IA81ORD send no response */
/*                - if input MessageType is IA81ASKP send IA81PRI    */
/*                     with IA81PRI.ItemUnitPrice set to a           */
/*                     randomly generated value                      */ 
/*                - else put an error message to stdout and terminate*/
/*                                                                   */
/*********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "legacyservice.h"
#include "cmqc.h"

#define WAIT_INTERVAL   1000 * 60 * 30      /* 30 minutes               */

   MQHCONN  HConn = MQHC_DEF_HCONN;         /* Connection handle        */
   MQHOBJ   HObj_CheckQ;                    /* Object handle            */
   MQLONG   CompCode;                       /* Completion code          */
   MQLONG   Reason;                         /* Qualifying reason        */
   MQOD     ObjDesc    = {MQOD_DEFAULT};    /* Object descriptor        */
   MQMD     MsgDesc    = {MQMD_DEFAULT};    /* Message descriptor       */
   MQLONG   OpenOptions;                    /* Control the MQOPEN call  */

   MQGMO    GetMsgOpts = {MQGMO_DEFAULT};   /* Get-Message Options      */
   MQLONG   MsgBuffLen;                     /* Length of message buffer */
   IA81BUY  MsgBuffer;                      /* Message structure        */
   IA81ASKP MsgBuffer2;                     /* Message structure        */
   MQLONG   DataLen;                        /* Length of message        */

   MQPMO    PutMsgOpts = {MQPMO_DEFAULT};   /* Put-Message Options      */
   IA81CONF PutBuffer;                      /* Message structure        */
   MQLONG   PutBuffLen = sizeof(PutBuffer);
                                         /* Length of message buffer */
   IA81PRI  PutBuffer2;                     /* Message structure        */
   MQLONG   PutBuff2Len = sizeof(PutBuffer2);
                                         /* Length of message buffer */

/*   function prototypes                                             */
void usageError( char* programName );
void errorMessage( char* msgStr, MQLONG CC, MQLONG RC );
void ProcessOrder(void);
void ProcessQuote(void);
void FindUnitPrice(void);


MQLONG main( int argc, char** argv )
{

   int flag = 0;		
   /* parameter variables                                            */
   MQCHAR48 qMgr;
   MQCHAR48 qName;
   
   memset( qMgr,  '\0', MQ_Q_MGR_NAME_LENGTH );
   memset( qName, '\0', MQ_Q_NAME_LENGTH );

   
   /*                                                                */
   /*  Move parameters into corresponding variables                  */
   /*                                                                */
   switch( argc )
      {
      case  3 : strncpy( qName, argv??( 2 ??), MQ_Q_NAME_LENGTH );
                strncpy( qMgr,  argv??( 1 ??), MQ_Q_MGR_NAME_LENGTH );
                break;
      default : usageError( argv??( 0 ??) );
                return 8;
      }

   /*                                                                */
   /*  Display the parameters to be used in the program              */
   /*                                                                */
   printf( "===========================================\n" );
   printf( "PARAMETERS PASSED :\n" );
   printf( "   QMGR        - %s\n", qMgr );
   printf( "   QNAME       - %s\n", qName );
   printf( "===========================================\n" );

   /*                                                                */
   /*   Connect to Queue Manager (MQCONN)                            */
   /*                                                                */
   MQCONN( qMgr,
           &HConn,
           &CompCode,
           &Reason     );
   /*                                                                */
   /*  If connect failed then display error message and exit         */
   /*                                                                */
   if( MQCC_OK != CompCode )
      {
      errorMessage( "MQCONN", CompCode, Reason );
      return Reason;
      }

   printf( "MQCONN SUCCESSFUL\n" );

   /*                                                                */
   /* Initialize options and open the queue for input                */
   /*                                                                */

   OpenOptions = MQOO_INPUT_SHARED +
                 MQOO_SAVE_ALL_CONTEXT;

   strncpy(ObjDesc.ObjectName, qName, MQ_Q_NAME_LENGTH);

   MQOPEN(HConn,
          &ObjDesc,
          OpenOptions,
          &HObj_CheckQ,
          &CompCode,
          &Reason);

   /*                                                                */
   /* Test the output from the open, if not OK then build an error   */
   /* message                                                        */
   /*                                                                */

   if (CompCode != MQCC_OK)
      errorMessage( "MQOPEN", CompCode, Reason );
   else
      {
      /*                                                             */
      /* Get and process messages                                    */
      /*                                                             */
      printf( "MQOPEN SUCCESSFUL\n" );

      GetMsgOpts.Options = MQGMO_WAIT +
                           MQGMO_FAIL_IF_QUIESCING +
						   MQGMO_ACCEPT_TRUNCATED_MSG +
                           MQGMO_SYNCPOINT;
      GetMsgOpts.WaitInterval = WAIT_INTERVAL;
      MsgBuffLen = sizeof(MsgBuffer);

      memcpy(MsgDesc.MsgId, MQMI_NONE, sizeof(MsgDesc.MsgId));
      memcpy(MsgDesc.CorrelId, MQCI_NONE, sizeof(MsgDesc.CorrelId));

      /*                                                             */
      /* Make the first get call outside the loop                    */
      /*                                                             */

      MQGET(HConn,
            HObj_CheckQ,
            &MsgDesc,
            &GetMsgOpts,
            MsgBuffLen,
            &MsgBuffer,
            &DataLen,
            &CompCode,
            &Reason);

      /*                                                             */
      /* Test the output of the get call inside the do-while loop.   */
      /* Loop until the get call fails.                              */
      /*                                                             */

      while (CompCode != MQCC_FAILED)
         {
         /*                                                          */
         /* Handle the message                                       */
         /*                                                          */
         printf( "MQGET SUCCESSFUL\n" );

         if (strncmp(MsgBuffer.MessageId,
                     BUY_MESSAGE,
                     sizeof(MsgBuffer.MessageId)) == 0)
		 {
            ProcessOrder();
			flag = 1;
		 }
         if (strncmp(MsgBuffer.MessageId,
                     ORDER_MESSAGE,
                     sizeof(MsgBuffer.MessageId)) == 0)
            flag = 1;  /* do nothing */ 
		 if (strncmp(MsgBuffer.MessageId,
                     REQUEST_PRICE,
                     sizeof(MsgBuffer.MessageId)) == 0)
		 {
            memcpy(&MsgBuffer2, &MsgBuffer, sizeof(MsgBuffer2));
            ProcessQuote();
			flag = 1;
		 }
		 /* else report an error as message is not ours */
         if ( flag != 1 ) 
		 {
           errorMessage( "Wrong message type", CompCode, Reason );
		   flag = 0;  /* reset the flag for next time */
         }


		 MQCMIT( HConn, &CompCode, &Reason );
         /*                                                             */
         /*   If commit failed then display error message               */
         /*                                                             */
         if( MQCC_OK != CompCode )
		 {
           errorMessage( "MQCMIT", CompCode, Reason );
         } /* else */    ;

         /*                                                          */
         /* Reset parameters for the next get call                   */
         /*                                                          */

         memcpy(MsgDesc.MsgId, MQMI_NONE, sizeof(MsgDesc.MsgId));
         memcpy(MsgDesc.CorrelId, MQCI_NONE, sizeof(MsgDesc.CorrelId));

         MQGET(HConn,
               HObj_CheckQ,
               &MsgDesc,
               &GetMsgOpts,
               MsgBuffLen,
               &MsgBuffer,
               &DataLen,
               &CompCode,
               &Reason);

         } /* end loop */

      /*                                                             */
      /* Test the output of the get call.  If the call failed,       */
      /* send an error message showing the completion code and       */
      /* reason code, unless the reason code is NO_MSG_AVAILABLE.    */
      /*                                                             */

      if (Reason != MQRC_NO_MSG_AVAILABLE)
         {
           errorMessage( "MQGET ", CompCode, Reason );
         }
         

      MQCLOSE(HConn,
              &HObj_CheckQ,
              MQCO_NONE,
              &CompCode,
              &Reason);

      if (CompCode != MQCC_OK)
      {
           errorMessage( "MQCLOSE", CompCode, Reason );
	  } 
   }
	return 0;
}

	  
	  
/*********************************************************************/
/*  This function develops a reply message and puts it on the        */
/*  reply queue, as specified in the input message.  If this         */
/*  operation fails, report the error and terminate                  */
/*********************************************************************/

void ProcessOrder(void)
   {
   /*                                                                */
   /* Build the reply message                                        */
   /*                                                                */

   memset(&PutBuffer, ' ', sizeof(PutBuffer));
   memcpy(&PutBuffer, &MsgBuffer, sizeof(MsgBuffer));
   strncpy(PutBuffer.MessageId, CONFIRMATION_MESSAGE, sizeof(PutBuffer.MessageId));

   /* decide whether to accept the order                             */

   if ( strchr( &PutBuffer.ItemReference, 'Y' ) != NULL )
   {
	   memset ( &PutBuffer.Confirm , 'Y', sizeof(PutBuffer.Confirm) ); 
   }
   else
	   memset ( &PutBuffer.Confirm , 'N', sizeof(PutBuffer.Confirm) );

   /*                                                                */
   /* Set the object descriptor, message descriptor, and put-        */
   /* message options to the values required to create the           */
   /* reply message.                                                 */
   /*                                                                */

   strncpy(ObjDesc.ObjectName, MsgDesc.ReplyToQ, MQ_Q_NAME_LENGTH);
   strncpy(ObjDesc.ObjectQMgrName, MsgDesc.ReplyToQMgr,
           MQ_Q_MGR_NAME_LENGTH);

   MsgDesc.MsgType = MQMT_REPLY;
   MsgDesc.Report  = MQRO_NONE;
   memset(MsgDesc.ReplyToQ, ' ', MQ_Q_NAME_LENGTH);
   memset(MsgDesc.ReplyToQMgr, ' ', MQ_Q_MGR_NAME_LENGTH);

   memcpy( MsgDesc.CorrelId, MsgDesc.MsgId, sizeof(MsgDesc.MsgId));
   memcpy( MsgDesc.MsgId, MQMI_NONE, sizeof(MsgDesc.MsgId));
   strncpy(MsgDesc.Format, MQFMT_STRING, MQ_FORMAT_LENGTH);

   PutMsgOpts.Options = MQPMO_SYNCPOINT +
                        MQPMO_PASS_IDENTITY_CONTEXT;
   PutMsgOpts.Context = HObj_CheckQ;

   PutBuffLen = sizeof(PutBuffer);

   MQPUT1(HConn,
          &ObjDesc,
          &MsgDesc,
          &PutMsgOpts,
          PutBuffLen,
          &PutBuffer,
          &CompCode,
          &Reason);


   if (CompCode != MQCC_OK)
      {
        errorMessage( "MQPUT1", CompCode, Reason );
      }

   printf( "MQPUT1 SUCCESSFUL\n" );
   }


/*********************************************************************/
/*  This function develops a reply message and puts it on the        */
/*  reply queue, as specified in the input message.  If this         */
/*  operation fails, report the error and terminate                  */
/*********************************************************************/

void ProcessQuote(void)
   {
   /*                                                                */
   /* Build the reply message                                        */
   /*                                                                */

   memset(&PutBuffer2, ' ', sizeof(PutBuffer2));
   memcpy(&PutBuffer2, &MsgBuffer2, sizeof(MsgBuffer2));
   strncpy(PutBuffer2.MessageId, PROVIDE_PRICE, sizeof(PutBuffer2.MessageId));

   /* Generate a price quotation                                     */ 
   FindUnitPrice() ;

   /*                                                                */
   /* Set the object descriptor, message descriptor, and put-        */
   /* message options to the values required to create the           */
   /* reply message.                                                 */
   /*                                                                */

   strncpy(ObjDesc.ObjectName, MsgDesc.ReplyToQ, MQ_Q_NAME_LENGTH);
   strncpy(ObjDesc.ObjectQMgrName, MsgDesc.ReplyToQMgr,
           MQ_Q_MGR_NAME_LENGTH);

   MsgDesc.MsgType = MQMT_REPLY;
   MsgDesc.Report  = MQRO_NONE;
   memset(MsgDesc.ReplyToQ, ' ', MQ_Q_NAME_LENGTH);
   memset(MsgDesc.ReplyToQMgr, ' ', MQ_Q_MGR_NAME_LENGTH);

   memcpy( MsgDesc.CorrelId, MsgDesc.MsgId, sizeof(MsgDesc.MsgId));
   memcpy( MsgDesc.MsgId, MQMI_NONE, sizeof(MsgDesc.MsgId));
   strncpy(MsgDesc.Format, MQFMT_STRING, MQ_FORMAT_LENGTH);

   PutMsgOpts.Options = MQPMO_SYNCPOINT +
                        MQPMO_PASS_IDENTITY_CONTEXT;
   PutMsgOpts.Context = HObj_CheckQ;

   PutBuffLen = sizeof(PutBuffer);

   MQPUT1(HConn,
          &ObjDesc,
          &MsgDesc,
          &PutMsgOpts,
          PutBuff2Len,
          &PutBuffer2,
          &CompCode,
          &Reason);


   if (CompCode != MQCC_OK)
      {
        errorMessage( "MQPUT1", CompCode, Reason );
      }

   printf( "MQPUT1 SUCCESSFUL\n" );
   }

/*********************************************************************/
/*  This function computes a random credit index based on a number   */
/*  obtained by calling the rand() function supplied by the stdlib.h */
/*  library.  The resulting message line is added to the reply       */
/*  message.                                                         */
/*********************************************************************/

 void FindUnitPrice(void)
   {

   int    RandNum;
   time_t t1;
   struct tm *t2;

   /*                                                                */
   /* Generate the unitprice                                         */
   /*                                                                */
   /* The srand() function sets the point for producing a series of  */
   /* pseudo-random integers.  The time (in seconds) is used to seed */
   /* the generator.                                                 */
   /*                                                                */

   t1 = time(NULL);
   t2 = localtime(&t1);

   srand(t2 -> tm_sec);

   RandNum = (rand()) * 5;

   sprintf(PutBuffer2.ItemUnitPrice, "%8d", RandNum);
   
   return;
   }           



/*********************************************************************/
void errorMessage( char* msgStr, MQLONG CC, MQLONG RC )
{
   printf( "************************************************\n" );
   printf( "* %s\n", msgStr );
   printf( "* COMPLETION CODE : %09ld\n", CC );
   printf( "* REASON CODE     : %09ld\n", RC );
   printf( "************************************************\n" );
}


/*********************************************************************/
void usageError( char* programName )
{
   printf( "==================================================\n" );
   printf( "PARAMETERS FOR %s\n", programName );
   printf( "     QMGR        - QUEUE MANGER\n" );
   printf( "     QNAME       - QUEUE NAME\n" );
   printf( "==================================================\n" );
}

